///
/// @file
/// @details An entity within the Egg project.
///
/// <!-- Copyright (c) 2015 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#include "basket_entity.h"
#include "game_results.h"

//--------------------------------------------------------------------------------------------------------------------//

Egg::BasketEntity::BasketEntity(void) :
	tbGame::Entity("BasketEntity"),
	//mSprite(tbGraphics::theSpriteManager.GetSprite("concept_art", "solid_square"))
	mSprite("data/basket.png")
{
	//mSprite.SetScale(2.0f, 1.0f);

	mSprite.SetOrigin(tbGraphics::kAnchorCenter);
	mSprite.SetPosition(0.0f, -36.0f);
	AddGraphic(mSprite);

	AddBoundingBox(128.0f, 64.0f);
}

//--------------------------------------------------------------------------------------------------------------------//

Egg::BasketEntity::~BasketEntity(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::BasketEntity::OnAdded(void)
{
	tbGame::Entity::OnAdded();
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::BasketEntity::OnRemoved(void)
{
	tbGame::Entity::OnRemoved();
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::BasketEntity::OnSimulate(void)
{
	tbGame::Entity::OnSimulate();
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::BasketEntity::OnUpdate(const float deltaTime)
{
	tbGame::Entity::OnUpdate(deltaTime);

	int mouseX(0);
	int mouseY(0);
	tbApplication::Input::GetMousePosition(&mouseX, &mouseY);

	const float basketX = tbMath::Clamp(static_cast<float>(mouseX), 0.0f, tbGraphics::ScreenWidth());
	SetPosition(basketX, tbGraphics::ScreenHeight() - 50.0f);
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::BasketEntity::OnRender(void) const
{
	tbGame::Entity::OnRender();
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::BasketEntity::OnCollideWith(tbGame::Entity& otherEntity)
{
	tbGame::Entity::OnCollideWith(otherEntity);

	if (true == otherEntity.IsEntityOfType("EggEntity"))
	{
		otherEntity.GetEntityManager()->RemoveEntity(&otherEntity);
		Egg::GameResults::IncrementCurrentScore();
		tbAudio::theAudioManager.PlayEvent("audio_events", "egg_pickup");
	}
}

//--------------------------------------------------------------------------------------------------------------------//
