///
/// @file
/// @details An entity within the Egg project.
///
/// <!-- Copyright (c) 2015 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#include "egg_entity.h"
#include "game_results.h"
#include "gameplay_scene.h"

//--------------------------------------------------------------------------------------------------------------------//

Egg::EggEntity::EggEntity(float eggSpeed) :
	tbGame::Entity("EggEntity"),
	//mSprite(tbGraphics::theSpriteManager.GetSprite("concept_art", "solid_circle")),
	mSprite("data/egg.png"),
	mEggFallSpeed(eggSpeed)
{
	//mSprite.SetScale(0.25f);
	mSprite.SetOrigin(tbGraphics::kAnchorCenter);
	AddGraphic(mSprite);

	AddBoundingCircle(16.0f);
}

//--------------------------------------------------------------------------------------------------------------------//

Egg::EggEntity::~EggEntity(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::EggEntity::OnAdded(void)
{
	tbGame::Entity::OnAdded();
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::EggEntity::OnRemoved(void)
{
	tbGame::Entity::OnRemoved();
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::EggEntity::OnSimulate(void)
{
	tbGame::Entity::OnSimulate();

	tbMath::Vector2 position(GetPosition());
	position.y += mEggFallSpeed * kFixedTime;
	SetPosition(position);

	if (position.y > tbGraphics::ScreenHeight())
	{
		//Splat.
		tbAudio::theAudioManager.PlayEvent("audio_events", "egg_splat");
		GetEntityManager()->RemoveEntity(this);
		GameplayScene::MarkResetGame();
	}
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::EggEntity::OnUpdate(const float deltaTime)
{
	tbGame::Entity::OnUpdate(deltaTime);
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::EggEntity::OnRender(void) const
{
	tbGame::Entity::OnRender();
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::EggEntity::OnCollideWith(tbGame::Entity& otherEntity)
{
	tbGame::Entity::OnCollideWith(otherEntity);
}

//--------------------------------------------------------------------------------------------------------------------//
