///
/// @file
/// @details This scene is the gameplay scene for the Egg project.
///
/// <!-- Copyright (c) 2015 Tim Beaudet - All Rights Reserved -->
///-----------------------------------------------------------------------------------------------------------------///

#include "gameplay_scene.h"
#include "scene_manager.h"
#include "score_display.h"
#include "game_results.h"

#include "basket_entity.h"
#include "egg_entity.h"

#include "turtle_brains/graphics/unstable/tbu_gif_exporter.h"

//--------------------------------------------------------------------------------------------------------------------//

static bool isMarkedForReset(false);
void Egg::GameplayScene::MarkResetGame(void)
{
	isMarkedForReset = true;
}

//--------------------------------------------------------------------------------------------------------------------//

Egg::GameplayScene::GameplayScene(void) :
	mNextEggDropTime(0)
{
	AddGraphic(new tbGraphics::Sprite("data/backdrop.png"));
}

//--------------------------------------------------------------------------------------------------------------------//

Egg::GameplayScene::~GameplayScene(void)
{
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::GameplayScene::OnSimulate(void)
{
	tbGame::GameScene::OnSimulate();

	if (true == mEggDropTimer.DecrementStep())
	{
		if (mNextEggDropTime > 2000)
		{
			mNextEggDropTime -= 150;
		}
		else if (mNextEggDropTime > 1500)
		{
			mNextEggDropTime -= 100;
		}
		else if (mNextEggDropTime > 500)
		{
			mNextEggDropTime -= 50;
		}
		else if (mNextEggDropTime > 250)
		{
			mNextEggDropTime -= 10;
		}

		mEggDropTimer = tbGame::GameTimer(mNextEggDropTime);
		
		EggEntity* egg = new EggEntity(tbMath::RandomFloat(100.0f, 300.0f));
		egg->SetPosition(tbMath::RandomFloat(10.0f, tbGraphics::ScreenWidth() - 10.0f), 0.0f);
		AddEntity(egg);
	}

	if (true == isMarkedForReset)
	{
		ResetGame();
	}
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::GameplayScene::OnUpdate(const float deltaTime)
{
	tbGame::GameScene::OnUpdate(deltaTime);

	if (true == tbApplication::Input::IsKeyReleased(tbApplication::tbKeyEscape))
	{	//This should be the very last check to happen as game ends immediately.
		theSceneManager->QuitGame();
	}

	tbGraphics::Unstable::GifPerformCapture(deltaTime, tbApplication::Input::IsKeyReleased(tbApplication::tbKeyG));
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::GameplayScene::OnRender(void) const
{
	tbGame::GameScene::OnRender();
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::GameplayScene::OnOpen(void)
{
	tbGame::GameScene::OnOpen();

	ResetGame();
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::GameplayScene::ResetGame(void)
{
	isMarkedForReset = false;

	RemoveEntities();

	AddEntity(new BasketEntity());
	AddEntity(new Egg::ScoreDisplay());

	Egg::GameResults::ResetGameResults(false);
	mNextEggDropTime = 3000;
	mEggDropTimer = 1;

	tbAudio::theAudioManager.PlayEvent("audio_events", "game_start");
}

//--------------------------------------------------------------------------------------------------------------------//

void Egg::GameplayScene::OnClose(void)
{
	tbGame::GameScene::OnClose();
}

//--------------------------------------------------------------------------------------------------------------------//
