
/// @file
///
/// @details This file contains implementation details specific to TurtleBrains and may be modified without warning.
/// @note using any classes, functions or definitions found within TurtleBrains::Implementation / tbImplementation is
///   not recommended as they can be changed or removed completely without warning.  This is your final warning.
///
/// <!-- Copyright (c) Tim Beaudet 2016 - All Rights Reserved -->
///------------------------------------------------------------------------------------------------------------------///

#include "../../../core/tb_configuration.h"
#ifdef tb_macosx

#include "../tbi_system_application_message_box.h"
#include "../../../core/tb_defines.h"
#include "../../../core/tb_error.h"
#include "../../tb_application_menu.h"
#include "../../tb_application_handler_interface.h"

#include <string>

#import <Foundation/NSString.h>
#import <Foundation/NSData.h>
#import <AppKit/NSAlert.h>
#import <AppKit/NSSavePanel.h>
#import <AppKit/NSOpenPanel.h>

//--------------------------------------------------------------------------------------------------------------------//

namespace tbImplementation
{
	//Defined in tbi_mac_application_dialog.cpp for now...
	NSString* ToCocoaString(const std::wstring& input);
	NSString* ToCocoaString(const std::string& input);
	std::wstring ToWideString(NSString* input);

};	/* namespace tbImplementation */

//--------------------------------------------------------------------------------------------------------------------//

bool tbImplementation::ModalMessageBox(const tbCore::tbString& title, const tbCore::tbString& message,
	const tbCore::tbString& primaryButton, const tbCore::tbString& secondaryButton)
{
	NSString *cocoaTitle = ToCocoaString(tbCore::ToWideString(title));
	NSString *cocoaMessage = ToCocoaString(tbCore::ToWideString(message));
	NSString *cocoaPrimary = ToCocoaString(tbCore::ToWideString(primaryButton));
	NSString *cocoaSecondary = (true == secondaryButton.empty()) ? nil : ToCocoaString(tbCore::ToWideString(secondaryButton));

	//NSRunAlertPanel(cocoaTitle, cocoaMessage, cocoaPrimary, cocoaSecondary, nil );

//	NSAlert* messageBox = [[NSAlert alloc] init];
//	messageBox.messageText = cocoaMessage;
//	messageBox.informativeText = cocoaTitle;
//	[messageBox.buttons objectAtIndex:0];
//	[messageBox getBu]

	NSAlert* messageBox = [NSAlert alertWithMessageText:cocoaTitle defaultButton:cocoaPrimary alternateButton:cocoaSecondary otherButton:nil
														informativeTextWithFormat:@"%@", cocoaMessage];

	NSInteger messageBoxResult = [messageBox runModal];
	return (true == secondaryButton.empty() || (NSAlertFirstButtonReturn != messageBoxResult)) ? true : false;
}

//--------------------------------------------------------------------------------------------------------------------//

bool tbImplementation::ModalOpenFileDialog(tbCore::tbString& fileToOpen, const tbCore::tbString& startDirectory,
	const tbCore::tbString& fileExtension)
{
	NSOpenPanel* openFileDialog = [NSOpenPanel openPanel];

	[openFileDialog setAllowsMultipleSelection:NO];
	[openFileDialog setCanChooseDirectories:YES]; //I assume this means can navigate directories.
	[openFileDialog setCanChooseFiles:YES];
	[openFileDialog setFloatingPanel:YES];

	NSInteger result = [openFileDialog runModal];

	bool didOpenFile = false;
	if (NSModalResponseOK == result)
	{
		fileToOpen = tbCore::ToString(ToWideString([[[openFileDialog URLs] objectAtIndex:0] absoluteString]));
		didOpenFile = true;
	}

	return didOpenFile;
}

//--------------------------------------------------------------------------------------------------------------------//

bool tbImplementation::ModalSaveFileDialog(tbCore::tbString& fileToOpen, const tbCore::tbString& startDirectory,
																					 const tbCore::tbString& fileExtension)
{
	NSSavePanel* saveFileDialog = [NSSavePanel savePanel];
	[saveFileDialog setFloatingPanel:YES];

	NSInteger result = [saveFileDialog runModal];

	bool didSaveFile = false;
	if (NSModalResponseOK == result)
	{
		fileToOpen = tbCore::ToString(ToWideString([[saveFileDialog URL] absoluteString]));
		didSaveFile = true;
	}

	return didSaveFile;
}

//--------------------------------------------------------------------------------------------------------------------//

#endif /* tb_macosx */
